#include "timelinedaymodel.h"
#include <QVariant>

TimelineDayModel::TimelineDayModel(QObject *parent)
    : QAbstractListModel(parent)
    , m_fromDate()
    , m_toDate()
{}

int TimelineDayModel::rowCount(const QModelIndex &parent) const
{
    if (parent.isValid()) {
        return 0;
    }
    return m_fromDate.daysTo(m_toDate) + 1;
}

QVariant TimelineDayModel::data(const QModelIndex &index, int role) const
{
    if (!index.isValid()) {
        return {};
    }

    QDate date = m_fromDate.addDays(index.row());
    switch (role) {
    case Qt::DisplayRole:
        return QString::number(date.day());
    }

    return {};
}

QDate TimelineDayModel::fromDate() const
{
    return m_fromDate;
}

void TimelineDayModel::setFromDate(QDate date)
{
    if (date == m_fromDate) {
        return;
    }
    beginResetModel();
    m_fromDate = date;
    endResetModel();
    emit fromDateChanged(m_fromDate);
}

QDate TimelineDayModel::toDate() const
{
    return m_toDate;
}

void TimelineDayModel::setToDate(QDate date)
{
    if (date == m_toDate) {
        return;
    }
    beginResetModel();
    m_toDate = date;
    endResetModel();
    emit toDateChanged(m_toDate);
}

#include "moc_timelinedaymodel.cpp"
