#ifndef TIMELINEVIEW_H
#define TIMELINEVIEW_H

#include <QList>
#include <QQmlComponent>
#include <QQuickItem>
#include "timelinemodel.h"

class TimelineView : public QQuickItem
{
    Q_OBJECT
    QML_ELEMENT
    Q_PROPERTY(qreal dayWidth READ dayWidth WRITE setDayWidth NOTIFY dayWidthChanged)
    Q_PROPERTY(QQmlComponent *delegate READ delegate WRITE setDelegate NOTIFY delegateChanged)
    Q_PROPERTY(QDate fromDate READ fromDate WRITE setFromDate NOTIFY fromDateChanged)
    Q_PROPERTY(QDate toDate READ toDate WRITE setToDate NOTIFY toDateChanged)
    Q_PROPERTY(TimelineModel *model READ model WRITE setModel NOTIFY modelChanged)
    Q_PROPERTY(qreal viewportX READ viewportX WRITE setViewportX NOTIFY viewportXChanged)
    Q_PROPERTY(
        qreal viewportWidth READ viewportWidth WRITE setViewportWidth NOTIFY viewportWidthChanged)

public:
    TimelineView(QQuickItem *parent = nullptr);
    ~TimelineView() override;

    qreal dayWidth() const;
    void setDayWidth(qreal width);

    QQmlComponent *delegate() const;
    void setDelegate(QQmlComponent *delegate);

    QDate fromDate() const;
    void setFromDate(QDate date);

    QDate toDate() const;
    void setToDate(QDate date);

    TimelineModel *model() const;
    void setModel(TimelineModel *model);

    qreal viewportX() const;
    void setViewportX(qreal x);

    qreal viewportWidth() const;
    void setViewportWidth(qreal width);

signals:
    void dayWidthChanged(qreal width);
    void delegateChanged(QQmlComponent *delegate);
    void fromDateChanged(QDate date);
    void toDateChanged(QDate date);
    void modelChanged(TimelineModel *model);
    void viewportXChanged(qreal x);
    void viewportWidthChanged(qreal width);

protected:
    void componentComplete() override;

private:
    Q_DISABLE_COPY_MOVE(TimelineView)

    struct Item;

    Item *createItem(qint64 day, const TimelineModel::Item &modelItem);
    void releaseItem(Item *item);
    void drainItems();

    void clear();
    void populate();
    void updateImplicitWidth();

    qreal m_dayWidth;
    QQmlComponent *m_delegate;
    QDate m_fromDate;
    QDate m_toDate;
    TimelineModel *m_model;
    QList<Item *> m_items;
    QList<QQuickItem *> m_reusableItems;
    qreal m_viewportX;
    qreal m_prevViewportX;
    qreal m_viewportWidth;
    qreal m_prevViewportWidth;
};

#endif // TIMELINEVIEW_H
