-- Test contact_phone
set client_min_messages to warning;
create extension if not exists pgtap;
reset client_min_messages;

begin;

select plan(21);

set search_path to camper, auth, public;

select has_table('contact_phone');
select has_pk('contact_phone' );
select table_privs_are('contact_phone', 'guest', array []::text[]);
select table_privs_are('contact_phone', 'employee', array ['SELECT', 'INSERT', 'UPDATE', 'DELETE']);
select table_privs_are('contact_phone', 'admin', array ['SELECT', 'INSERT', 'UPDATE', 'DELETE']);
select table_privs_are('contact_phone', 'authenticator', array []::text[]);

select has_column('contact_phone', 'contact_id');
select col_is_pk('contact_phone', 'contact_id');
select col_is_fk('contact_phone', 'contact_id');
select fk_ok('contact_phone', 'contact_id', 'contact', 'contact_id');
select col_type_is('contact_phone', 'contact_id', 'integer');
select col_not_null('contact_phone', 'contact_id');
select col_hasnt_default('contact_phone', 'contact_id');

select has_column('contact_phone', 'phone');
select col_type_is('contact_phone', 'phone', 'packed_phone_number');
select col_not_null('contact_phone', 'phone');
select col_hasnt_default('contact_phone', 'phone');


set client_min_messages to warning;
truncate contact_phone cascade;
truncate contact cascade;
truncate company_host cascade;
truncate company_user cascade;
truncate company cascade;
truncate auth."user" cascade;
reset client_min_messages;

insert into auth."user" (user_id, email, name, password, cookie, cookie_expires_at)
values (1, 'demo@tandem.blog', 'Demo', 'test', '44facbb30d8a419dfd4bfbc44a4b5539d4970148dfc84bed0e', current_timestamp + interval '1 month')
     , (5, 'admin@tandem.blog', 'Demo', 'test', '12af4c88b528c2ad4222e3740496ecbc58e76e26f087657524', current_timestamp + interval '1 month')
;

insert into company (company_id, business_name, vatin, trade_name, phone, email, web, address, city, province, postal_code, rtc_number, tourist_tax, tourist_tax_max_days, country_code, currency_code, default_lang_tag)
values (2, 'Company 2', 'XX123', '', '555-555-555', 'a@a', '', '', '', '', '', '', 60, 7, 'ES', 'EUR', 'ca')
     , (4, 'Company 4', 'XX234', '', '666-666-666', 'b@b', '', '', '', '', '', '', 60, 7, 'FR', 'USD', 'ca')
;

insert into company_user (company_id, user_id, role)
values (2, 1, 'admin')
     , (4, 5, 'admin')
;

insert into company_host (company_id, host)
values (2, 'co2')
     , (4, 'co4')
;

insert into contact (contact_id, company_id, name, id_document_type_id, id_document_number, address, city, province, postal_code, country_code)
values (6, 2, 'C1', 'D', '41440443Q', 'Fake St', 'City', 'Province', '17600', 'ES')
     , (8, 4, 'C2', 'D', '41440443Q', 'Fake St', 'City', 'Province', '17600', 'ES')
     , (9, 4, 'C3', 'D', '41440443Q', 'Fake St', 'City', 'Province', '17600', 'ES')
;

insert into contact_phone (contact_id, phone)
values (6, '777-777-777')
     , (8, '888-888-888')
;

prepare contact_data as
select company_id, phone
from contact
join contact_phone using (contact_id)
order by company_id, phone;

set role employee;
select is_empty('contact_data', 'Should show no data when cookie is not set yet');
reset role;

select set_cookie('44facbb30d8a419dfd4bfbc44a4b5539d4970148dfc84bed0e/demo@tandem.blog', 'co2');
select bag_eq(
	'contact_data',
	$$ values (2, '777-777-777'::packed_phone_number)
	$$,
	'Should only list contacts of the companies where demo@tandem.blog is user of'
);
reset role;

select set_cookie('12af4c88b528c2ad4222e3740496ecbc58e76e26f087657524/admin@tandem.blog', 'co4');
select bag_eq(
	'contact_data',
	$$ values (4, '888-888-888'::packed_phone_number)
	$$,
	'Should only list contacts of the companies where admin@tandem.blog is user of'
);
reset role;

select set_cookie('not-a-cookie', 'co4');
select throws_ok(
	'contact_data',
	'42501', 'permission denied for table contact',
	'Should not allow select to guest users'
);
reset role;


select *
from finish();

rollback;

